@@ -1,6 +1,8 @@
 #ifndef ASN1_H
 #define ASN1_H
 
+#include <net-snmp/library/oid.h>
+
 #ifdef __cplusplus
 extern          "C" {
 #endif
@@ -35,17 +37,11 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 ******************************************************************/
-
-
-#ifndef MAX_SUBID               /* temporary - duplicate definition protection */
-#ifndef EIGHTBIT_SUBIDS
-    typedef u_long  oid;
-#define MAX_SUBID   0xFFFFFFFF
-#else
-    typedef u_char  oid;
-#define MAX_SUBID   0xFF
-#endif
-#endif
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ */
 
 #define MIN_OID_LEN	    2
 #define MAX_OID_LEN	    128 /* max subid's in an oid */
@@ -56,26 +52,26 @@ SOFTWARE.
 #define OID_LENGTH(x)  (sizeof(x)/sizeof(oid))
 
 
-#define ASN_BOOLEAN	    ((u_char)0x01)
-#define ASN_INTEGER	    ((u_char)0x02)
-#define ASN_BIT_STR	    ((u_char)0x03)
-#define ASN_OCTET_STR	    ((u_char)0x04)
-#define ASN_NULL	    ((u_char)0x05)
-#define ASN_OBJECT_ID	    ((u_char)0x06)
-#define ASN_SEQUENCE	    ((u_char)0x10)
-#define ASN_SET		    ((u_char)0x11)
+#define ASN_BOOLEAN	    0x01U
+#define ASN_INTEGER	    0x02U
+#define ASN_BIT_STR	    0x03U
+#define ASN_OCTET_STR	    0x04U
+#define ASN_NULL	    0x05U
+#define ASN_OBJECT_ID	    0x06U
+#define ASN_SEQUENCE	    0x10U
+#define ASN_SET		    0x11U
 
-#define ASN_UNIVERSAL	    ((u_char)0x00)
-#define ASN_APPLICATION     ((u_char)0x40)
-#define ASN_CONTEXT	    ((u_char)0x80)
-#define ASN_PRIVATE	    ((u_char)0xC0)
+#define ASN_UNIVERSAL	    0x00U
+#define ASN_APPLICATION     0x40U
+#define ASN_CONTEXT	    0x80U
+#define ASN_PRIVATE	    0xC0U
 
-#define ASN_PRIMITIVE	    ((u_char)0x00)
-#define ASN_CONSTRUCTOR	    ((u_char)0x20)
+#define ASN_PRIMITIVE	    0x00U
+#define ASN_CONSTRUCTOR	    0x20U
 
-#define ASN_LONG_LEN	    (0x80)
-#define ASN_EXTENSION_ID    (0x1F)
-#define ASN_BIT8	    (0x80)
+#define ASN_LONG_LEN	    0x80U
+#define ASN_EXTENSION_ID    0x1FU
+#define ASN_BIT8	    0x80U
 
 #define IS_CONSTRUCTOR(byte)	((byte) & ASN_CONSTRUCTOR)
 #define IS_EXTENSION_ID(byte)	(((byte) & ASN_EXTENSION_ID) == ASN_EXTENSION_ID)
@@ -85,7 +81,7 @@ SOFTWARE.
         u_long          low;
     };
 
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
     typedef struct counter64 integer64;
     typedef struct counter64 unsigned64;
 
@@ -111,9 +107,9 @@ SOFTWARE.
      * base value for the second octet of the tag - the
      * second octet was the value for the tag 
      */
-#define ASN_OPAQUE_TAG2 ((u_char)0x30)
+#define ASN_OPAQUE_TAG2 0x30U
 
-#define ASN_OPAQUE_TAG2U ((u_char)0x2f) /* second octet of tag for union */
+#define ASN_OPAQUE_TAG2U 0x2fU /* second octet of tag for union */
 
     /*
      * All the ASN.1 types for SNMP "should have been" defined in this file,
@@ -173,7 +169,7 @@ SOFTWARE.
      */
 #define ASN_OPAQUE_U64_MX_BER_LEN 12
 
-#endif                          /* OPAQUE_SPECIAL_TYPES */
+#endif                          /* NETSNMP_WITH_OPAQUE_SPECIAL_TYPES */
 
 
 #define ASN_PRIV_INCL_RANGE (ASN_PRIVATE | 2)
@@ -181,63 +177,92 @@ SOFTWARE.
 #define ASN_PRIV_DELEGATED  (ASN_PRIVATE | 5)
 #define ASN_PRIV_IMPLIED_OCTET_STR  (ASN_PRIVATE | ASN_OCTET_STR)       /* 4 */
 #define ASN_PRIV_IMPLIED_OBJECT_ID  (ASN_PRIVATE | ASN_OBJECT_ID)       /* 6 */
-#define ASN_PRIV_RETRY      (ASN_PRIVATE | 7)
+#define ASN_PRIV_RETRY      (ASN_PRIVATE | 7)   /* 199 */
+#define ASN_PRIV_STOP       (ASN_PRIVATE | 8)   /* 200 */
 #define IS_DELEGATED(x)   ((x) == ASN_PRIV_DELEGATED)
 
 
+    NETSNMP_IMPORT
     int             asn_check_packet(u_char *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_int(u_char *, size_t *, u_char *, long *,
                                   size_t);
-    u_char         *asn_build_int(u_char *, size_t *, u_char, long *,
+    NETSNMP_IMPORT
+    u_char         *asn_build_int(u_char *, size_t *, u_char, const long *,
                                   size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_unsigned_int(u_char *, size_t *, u_char *,
                                            u_long *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_build_unsigned_int(u_char *, size_t *, u_char,
-                                           u_long *, size_t);
+                                           const u_long *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_string(u_char *, size_t *, u_char *,
                                      u_char *, size_t *);
+    NETSNMP_IMPORT
     u_char         *asn_build_string(u_char *, size_t *, u_char,
                                      const u_char *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_header(u_char *, size_t *, u_char *);
+    NETSNMP_IMPORT
     u_char         *asn_parse_sequence(u_char *, size_t *, u_char *, u_char expected_type,      /* must be this type */
                                        const char *estr);       /* error message prefix */
+    NETSNMP_IMPORT
     u_char         *asn_build_header(u_char *, size_t *, u_char, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_build_sequence(u_char *, size_t *, u_char, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_length(u_char *, u_long *);
+    NETSNMP_IMPORT
     u_char         *asn_build_length(u_char *, size_t *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_objid(u_char *, size_t *, u_char *, oid *,
                                     size_t *);
+    NETSNMP_IMPORT
     u_char         *asn_build_objid(u_char *, size_t *, u_char, oid *,
                                     size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_null(u_char *, size_t *, u_char *);
+    NETSNMP_IMPORT
     u_char         *asn_build_null(u_char *, size_t *, u_char);
+    NETSNMP_IMPORT
     u_char         *asn_parse_bitstring(u_char *, size_t *, u_char *,
                                         u_char *, size_t *);
+    NETSNMP_IMPORT
     u_char         *asn_build_bitstring(u_char *, size_t *, u_char,
-                                        u_char *, size_t);
+                                        const u_char *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_unsigned_int64(u_char *, size_t *, u_char *,
                                              struct counter64 *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_build_unsigned_int64(u_char *, size_t *, u_char,
-                                             struct counter64 *, size_t);
+                                             const struct counter64 *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_signed_int64(u_char *, size_t *, u_char *,
                                            struct counter64 *, size_t);
+    NETSNMP_IMPORT
     u_char         *asn_build_signed_int64(u_char *, size_t *, u_char,
-                                           struct counter64 *, size_t);
-    u_char         *asn_build_float(u_char *, size_t *, u_char, float *,
+                                           const struct counter64 *, size_t);
+    NETSNMP_IMPORT
+    u_char         *asn_build_float(u_char *, size_t *, u_char, const float *,
                                     size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_float(u_char *, size_t *, u_char *, float *,
                                     size_t);
-    u_char         *asn_build_double(u_char *, size_t *, u_char, double *,
+    NETSNMP_IMPORT
+    u_char         *asn_build_double(u_char *, size_t *, u_char, const double *,
                                      size_t);
+    NETSNMP_IMPORT
     u_char         *asn_parse_double(u_char *, size_t *, u_char *,
                                      double *, size_t);
 
-#ifdef USE_REVERSE_ASNENCODING
+#ifdef NETSNMP_USE_REVERSE_ASNENCODING
 
     /*
      * Re-allocator function for below.  
      */
 
+    NETSNMP_IMPORT
     int             asn_realloc(u_char **, size_t *);
 
     /*
@@ -345,11 +370,13 @@ SOFTWARE.
      */
 
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_int(u_char ** pkt, size_t * pkt_len,
                                            size_t * offset,
                                            int allow_realloc, u_char type,
-                                           long *data, size_t data_size);
+                                           const long *data, size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_string(u_char ** pkt,
                                               size_t * pkt_len,
                                               size_t * offset,
@@ -358,14 +385,16 @@ SOFTWARE.
                                               const u_char * data,
                                               size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_unsigned_int(u_char ** pkt,
                                                     size_t * pkt_len,
                                                     size_t * offset,
                                                     int allow_realloc,
                                                     u_char type,
-                                                    u_long * data,
+                                                    const u_long * data,
                                                     size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_header(u_char ** pkt,
                                               size_t * pkt_len,
                                               size_t * offset,
@@ -373,6 +402,7 @@ SOFTWARE.
                                               u_char type,
                                               size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_sequence(u_char ** pkt,
                                                 size_t * pkt_len,
                                                 size_t * offset,
@@ -380,12 +410,14 @@ SOFTWARE.
                                                 u_char type,
                                                 size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_length(u_char ** pkt,
                                               size_t * pkt_len,
                                               size_t * offset,
                                               int allow_realloc,
                                               size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_objid(u_char ** pkt,
                                              size_t * pkt_len,
                                              size_t * offset,
@@ -393,48 +425,54 @@ SOFTWARE.
                                              u_char type, const oid *,
                                              size_t);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_null(u_char ** pkt,
                                             size_t * pkt_len,
                                             size_t * offset,
                                             int allow_realloc,
                                             u_char type);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_bitstring(u_char ** pkt,
                                                  size_t * pkt_len,
                                                  size_t * offset,
                                                  int allow_realloc,
                                                  u_char type,
-                                                 u_char * data,
+                                                 const u_char * data,
                                                  size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_unsigned_int64(u_char ** pkt,
                                                       size_t * pkt_len,
                                                       size_t * offset,
                                                       int allow_realloc,
                                                       u_char type,
                                                       struct counter64
-                                                      *data, size_t);
+                                                      const *data, size_t);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_signed_int64(u_char ** pkt,
                                                     size_t * pkt_len,
                                                     size_t * offset,
                                                     int allow_realloc,
                                                     u_char type,
-                                                    struct counter64 *data,
+                                                    const struct counter64 *data,
                                                     size_t);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_float(u_char ** pkt,
                                              size_t * pkt_len,
                                              size_t * offset,
                                              int allow_realloc,
-                                             u_char type, float *data,
+                                             u_char type, const float *data,
                                              size_t data_size);
 
+    NETSNMP_IMPORT
     int             asn_realloc_rbuild_double(u_char ** pkt,
                                               size_t * pkt_len,
                                               size_t * offset,
                                               int allow_realloc,
-                                              u_char type, double *data,
+                                              u_char type, const double *data,
                                               size_t data_size);
 #endif
 
