@@ -1,3 +1,14 @@
+/*
+ * Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
+
 #ifndef NET_SNMP_SESSION_API_H
 #define NET_SNMP_SESSION_API_H
 
@@ -12,6 +23,10 @@
 extern          "C" {
 #endif
 
+    struct session_list;
+    struct timeval;
+
+
     NETSNMP_IMPORT
     void            snmp_sess_init(netsnmp_session *);
 
@@ -43,6 +58,10 @@ extern          "C" {
     NETSNMP_IMPORT
     int             snmp_close_sessions(void);
 
+    NETSNMP_IMPORT
+    int
+    _build_initial_pdu_packet(struct session_list *slp, netsnmp_pdu *pdu,
+                              int bulk);
 
     /*
      * int snmp_send(session, pdu)
@@ -54,6 +73,7 @@ extern          "C" {
      * session defaults.  Add a request corresponding to this pdu to the list
      * of outstanding requests on this session, then send the pdu.
      * Returns the request id of the generated packet if applicable, otherwise 1.
+     * (There is a special case: if the request id is 0, 1 will be returned).
      * On any error, 0 is returned.
      * The pdu is freed by snmp_send() unless a failure occured.
      */
@@ -147,12 +167,6 @@ extern          "C" {
     int             snmp_select_info2(int *, netsnmp_large_fd_set *,
                                       struct timeval *, int *);
 
-    /*
-     * snmp_sess_select_info_flags() and
-     * snmp_sess_select_info2_flags() is similar to
-     * snmp_sess_select_info() and snmp_sess_select_info2(), but
-     * accepts a list of flags to control aspects of its behavior.
-     */
 #define NETSNMP_SELECT_NOFLAGS  0x00
 #define NETSNMP_SELECT_NOALARMS 0x01
     NETSNMP_IMPORT
@@ -191,7 +205,6 @@ extern          "C" {
      
      snmp_sess_init(&session);
      session.retries = ...
-     session.remote_port = ...
      sessp = snmp_sess_open(&session);
      ss = snmp_sess_session(sessp);
      if (ss == NULL)
@@ -223,7 +236,8 @@ extern          "C" {
     netsnmp_session *snmp_sess_session(void *);
     NETSNMP_IMPORT
     netsnmp_session *snmp_sess_session_lookup(void *);
-
+    NETSNMP_IMPORT
+    netsnmp_session *snmp_sess_lookup_by_name(const char *paramName);
 
     /*
      * use return value from snmp_sess_open as void * parameter 
